#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .NamespaceAliasBuilder import NamespaceAliasBuilder
from .NamespaceIdDto import NamespaceIdDto
from .NamespaceLifetimeBuilder import NamespaceLifetimeBuilder
from .NamespacePathBuilder import NamespacePathBuilder
from .StateHeaderBuilder import StateHeaderBuilder

class RootNamespaceHistoryBuilder(StateHeaderBuilder):
    """Binary layout for non-historical root namespace history.

    Attributes:
        id: Id of the root namespace history.
        ownerAddress: Namespace owner address.
        lifetime: Lifetime in blocks.
        rootAlias: Root namespace alias.
        paths: Save child sub-namespace paths.
    """

    def __init__(self, version: int, id: NamespaceIdDto, ownerAddress: AddressDto, lifetime: NamespaceLifetimeBuilder, rootAlias: NamespaceAliasBuilder, paths: List[NamespacePathBuilder]):
        """Constructor.
        Args:
            version: Serialization version.
            id: Id of the root namespace history.
            ownerAddress: Namespace owner address.
            lifetime: Lifetime in blocks.
            rootAlias: Root namespace alias.
            paths: Save child sub-namespace paths.
        """
        super().__init__(version)
        self.id = id
        self.ownerAddress = ownerAddress
        self.lifetime = lifetime
        self.rootAlias = rootAlias
        self.paths = paths


    @classmethod
    def load_from_binary(cls, payload: bytes) -> RootNamespaceHistoryBuilder:
        """Creates an instance of RootNamespaceHistoryBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of RootNamespaceHistoryBuilder.
        """
        bytes_ = bytes(payload)
        superObject = StateHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        id = NamespaceIdDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[id.get_size():]
        ownerAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[ownerAddress.get_size():]
        lifetime = NamespaceLifetimeBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[lifetime.get_size():]
        rootAlias = NamespaceAliasBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[rootAlias.get_size():]
        childrenCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIZE_FIELD
        bytes_ = bytes_[8:]
        paths: List[NamespacePathBuilder] = []  # kind:ARRAY
        for _ in range(childrenCount):
            item = NamespacePathBuilder.load_from_binary(bytes_)
            paths.append(item)
            bytes_ = bytes_[item.get_size():]
        return RootNamespaceHistoryBuilder(superObject.version, id, ownerAddress, lifetime, rootAlias, paths)

    def get_id(self) -> NamespaceIdDto:
        """Gets id of the root namespace history.
        Returns:
            Id of the root namespace history.
        """
        return self.id

    def get_owner_address(self) -> AddressDto:
        """Gets namespace owner address.
        Returns:
            Namespace owner address.
        """
        return self.ownerAddress

    def get_lifetime(self) -> NamespaceLifetimeBuilder:
        """Gets lifetime in blocks.
        Returns:
            Lifetime in blocks.
        """
        return self.lifetime

    def get_root_alias(self) -> NamespaceAliasBuilder:
        """Gets root namespace alias.
        Returns:
            Root namespace alias.
        """
        return self.rootAlias

    def get_paths(self) -> List[NamespacePathBuilder]:
        """Gets save child sub-namespace paths.
        Returns:
            Save child sub-namespace paths.
        """
        return self.paths

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.id.get_size()
        size += self.ownerAddress.get_size()
        size += self.lifetime.get_size()
        size += self.rootAlias.get_size()
        size += 8  # childrenCount
        for _ in self.paths:
            size += _.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.id.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.ownerAddress.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.lifetime.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.rootAlias.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(len(self.get_paths()), 8))  # kind:SIZE_FIELD
        for _ in self.paths: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .Hash256Dto import Hash256Dto
from .LockHashAlgorithmDto import LockHashAlgorithmDto
from .UnresolvedAddressDto import UnresolvedAddressDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class SecretProofTransactionBodyBuilder:
    """Binary layout for a secret proof transaction.

    Attributes:
        recipient_address: Locked mosaic recipient address.
        secret: Secret.
        hash_algorithm: Hash algorithm.
        proof: Proof data.
    """
    def __init__(self):
        """ Constructor."""
        self.recipient_address = bytes(24)
        self.secret = bytes(32)
        self.hash_algorithm = LockHashAlgorithmDto(0).value
        self.proof = bytes()

    @classmethod
    def load_from_binary(cls, payload: bytes) -> SecretProofTransactionBodyBuilder:
        """Creates an instance of SecretProofTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of SecretProofTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        recipient_address_ = UnresolvedAddressDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        recipient_address = recipient_address_.unresolvedAddress
        bytes_ = bytes_[recipient_address_.get_size():]
        secret_ = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        secret = secret_.hash256
        bytes_ = bytes_[secret_.get_size():]
        proof_size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 2))  # kind:SIZE_FIELD
        bytes_ = bytes_[2:]
        hash_algorithm_ = LockHashAlgorithmDto.load_from_binary(bytes_)  # kind:CUSTOM2
        hash_algorithm = hash_algorithm_.value
        bytes_ = bytes_[hash_algorithm_.get_size():]
        proof = GeneratorUtils.get_bytes(bytes_, proof_size)  # kind:BUFFER
        bytes_ = bytes_[proof_size:]

        # create object and call
        result = SecretProofTransactionBodyBuilder()
        result.recipient_address = recipient_address
        result.secret = secret
        result.hash_algorithm = hash_algorithm
        result.proof = proof
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += UnresolvedAddressDto(self.recipient_address).get_size()
        size += Hash256Dto(self.secret).get_size()
        size += 2  # proof_size
        size += LockHashAlgorithmDto(self.hash_algorithm).get_size()
        size += len(self.proof)
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedAddressDto(self.recipient_address).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, Hash256Dto(self.secret).serialize())  # kind:CUSTOM
        size_value = len(self.proof)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 2))  # kind:SIZE_FIELD
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, LockHashAlgorithmDto(self.hash_algorithm).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.proof)  # kind:BUFFER
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('recipient_address', to_hex_string(UnresolvedAddressDto(self.recipient_address).serialize()))
        result += '{:24s} : {}\n'.format('secret', to_hex_string(Hash256Dto(self.secret).serialize()))
        size_value = len(self.proof)
        result += '{:24s} : {}\n'.format('proof_size', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 2)))
        result += '{:24s} : {}\n'.format('hash_algorithm', to_hex_string(LockHashAlgorithmDto(self.hash_algorithm).serialize()))
        result += '{:24s} : {}\n'.format('proof', to_hex_string(self.proof))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from binascii import hexlify
from .GeneratorUtils import GeneratorUtils

class FinalizationPointDto:
    """Finalization point.

    Attributes:
        finalizationPoint: Finalization point.
    """
    def __init__(self, finalizationPoint: int = 0):
        """Constructor.

        Args:
            finalizationPoint: Finalization point.
        """
        self.finalizationPoint = finalizationPoint

    @classmethod
    def load_from_binary(cls, payload: bytes) -> FinalizationPointDto:
        """Creates an instance of FinalizationPointDto from binary payload.

        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of FinalizationPointDto.
        """
        bytes_ = bytes(payload)
        finalizationPoint = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))
        return FinalizationPointDto(finalizationPoint)

    @classmethod
    def get_size(cls) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        return 4

    def get_finalization_point(self) -> int:
        """Gets Finalization point.

        Returns:
            Finalization point.
        """
        return self.finalizationPoint

    def serialize(self) -> bytes:
        """Serializes self to bytes.

        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_finalization_point(), 4))
        return bytes_

    def __str__(self):
        result = hexlify(GeneratorUtils.uint_to_buffer(self.get_finalization_point(), 4)).decode('utf-8')
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
from typing import List
import re
from .GeneratorUtils import GeneratorUtils
from .UnresolvedAddressDto import UnresolvedAddressDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class MultisigAccountModificationTransactionBodyBuilder:
    """Binary layout for a multisig account modification transaction.

    Attributes:
        min_removal_delta: Relative change of the minimal number of cosignatories required when removing an account.
        min_approval_delta: Relative change of the minimal number of cosignatories required when approving a transaction.
        address_additions: Cosignatory address additions.
        address_deletions: Cosignatory address deletions.
    """
    def __init__(self):
        """ Constructor."""
        self.min_removal_delta = int()
        self.min_approval_delta = int()
        self.address_additions = []
        self.address_deletions = []

    @classmethod
    def load_from_binary(cls, payload: bytes) -> MultisigAccountModificationTransactionBodyBuilder:
        """Creates an instance of MultisigAccountModificationTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MultisigAccountModificationTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        min_removal_delta = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]
        min_approval_delta = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]
        address_additions_count = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIZE_FIELD
        bytes_ = bytes_[1:]
        address_deletions_count = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIZE_FIELD
        bytes_ = bytes_[1:]
        multisig_account_modification_transaction_body__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        address_additions = []  # kind:ARRAY
        for _ in range(address_additions_count):
            item = UnresolvedAddressDto.load_from_binary(bytes_)
            address_additions.append(item.unresolvedAddress)
            bytes_ = bytes_[item.get_size():]
        address_deletions = []  # kind:ARRAY
        for _ in range(address_deletions_count):
            item = UnresolvedAddressDto.load_from_binary(bytes_)
            address_deletions.append(item.unresolvedAddress)
            bytes_ = bytes_[item.get_size():]

        # create object and call
        result = MultisigAccountModificationTransactionBodyBuilder()
        result.min_removal_delta = min_removal_delta
        result.min_approval_delta = min_approval_delta
        result.address_additions = address_additions
        result.address_deletions = address_deletions
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 1  # min_removal_delta
        size += 1  # min_approval_delta
        size += 1  # address_additions_count
        size += 1  # address_deletions_count
        size += 4  # multisig_account_modification_transaction_body__reserved1
        for _ in self.address_additions:
            size += UnresolvedAddressDto(_).get_size()
        for _ in self.address_deletions:
            size += UnresolvedAddressDto(_).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.min_removal_delta, 1))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.min_approval_delta, 1))  # serial_kind:SIMPLE
        size_value = len(self.address_additions)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 1))  # kind:SIZE_FIELD
        size_value = len(self.address_deletions)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 1))  # kind:SIZE_FIELD
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        for _ in self.address_additions: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedAddressDto(_).serialize())
        for _ in self.address_deletions: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedAddressDto(_).serialize())
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('min_removal_delta', to_hex_string(GeneratorUtils.uint_to_buffer(self.min_removal_delta, 1)))
        result += '{:24s} : {}\n'.format('min_approval_delta', to_hex_string(GeneratorUtils.uint_to_buffer(self.min_approval_delta, 1)))
        size_value = len(self.address_additions)
        result += '{:24s} : {}\n'.format('address_additions_count', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 1)))
        size_value = len(self.address_deletions)
        result += '{:24s} : {}\n'.format('address_deletions_count', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 1)))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        result += '{:24s} : [\n'.format('address_additions')
        for _ in self.address_additions: # kind:ARRAY|FILL_ARRAY
            result += '  {}\n'.format(UnresolvedAddressDto(_).__str__())
        result += ']\n'
        result += '{:24s} : [\n'.format('address_deletions')
        for _ in self.address_deletions: # kind:ARRAY|FILL_ARRAY
            result += '  {}\n'.format(UnresolvedAddressDto(_).__str__())
        result += ']\n'
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from enum import Enum
from .GeneratorUtils import GeneratorUtils


class EntityTypeDto(Enum):
    """Enumeration of entity types

    Attributes:
        RESERVED: reserved entity type.
        NEMESIS_BLOCK_HEADER: Nemesis block header.
        NORMAL_BLOCK_HEADER: Normal block header.
        IMPORTANCE_BLOCK_HEADER: Importance block header.
        ACCOUNT_KEY_LINK_TRANSACTION: Account key link transaction.
        EMBEDDED_ACCOUNT_KEY_LINK_TRANSACTION: Embedded account key link transaction.
        NODE_KEY_LINK_TRANSACTION: Node key link transaction.
        EMBEDDED_NODE_KEY_LINK_TRANSACTION: Embedded node key link transaction.
        AGGREGATE_COMPLETE_TRANSACTION: Aggregate complete transaction.
        AGGREGATE_BONDED_TRANSACTION: Aggregate bonded transaction.
        VOTING_KEY_LINK_TRANSACTION: Voting key link transaction.
        EMBEDDED_VOTING_KEY_LINK_TRANSACTION: Embedded voting key link transaction.
        VRF_KEY_LINK_TRANSACTION: Vrf key link transaction.
        EMBEDDED_VRF_KEY_LINK_TRANSACTION: Embedded vrf key link transaction.
        HASH_LOCK_TRANSACTION: Hash lock transaction.
        EMBEDDED_HASH_LOCK_TRANSACTION: Embedded hash lock transaction.
        SECRET_LOCK_TRANSACTION: Secret lock transaction.
        EMBEDDED_SECRET_LOCK_TRANSACTION: Embedded secret lock transaction.
        SECRET_PROOF_TRANSACTION: Secret proof transaction.
        EMBEDDED_SECRET_PROOF_TRANSACTION: Embedded secret proof transaction.
        ACCOUNT_METADATA_TRANSACTION: Account metadata transaction.
        EMBEDDED_ACCOUNT_METADATA_TRANSACTION: Embedded account metadata transaction.
        MOSAIC_METADATA_TRANSACTION: Mosaic metadata transaction.
        EMBEDDED_MOSAIC_METADATA_TRANSACTION: Embedded mosaic metadata transaction.
        NAMESPACE_METADATA_TRANSACTION: Namespace metadata transaction.
        EMBEDDED_NAMESPACE_METADATA_TRANSACTION: Embedded namespace metadata transaction.
        MOSAIC_DEFINITION_TRANSACTION: Mosaic definition transaction.
        EMBEDDED_MOSAIC_DEFINITION_TRANSACTION: Embedded mosaic definition transaction.
        MOSAIC_SUPPLY_CHANGE_TRANSACTION: Mosaic supply change transaction.
        EMBEDDED_MOSAIC_SUPPLY_CHANGE_TRANSACTION: Embedded mosaic supply change transaction.
        MULTISIG_ACCOUNT_MODIFICATION_TRANSACTION: Multisig account modification transaction.
        EMBEDDED_MULTISIG_ACCOUNT_MODIFICATION_TRANSACTION: Embedded multisig account modification transaction.
        ADDRESS_ALIAS_TRANSACTION: Address alias transaction.
        EMBEDDED_ADDRESS_ALIAS_TRANSACTION: Embedded address alias transaction.
        MOSAIC_ALIAS_TRANSACTION: Mosaic alias transaction.
        EMBEDDED_MOSAIC_ALIAS_TRANSACTION: Embedded mosaic alias transaction.
        NAMESPACE_REGISTRATION_TRANSACTION: Namespace registration transaction.
        EMBEDDED_NAMESPACE_REGISTRATION_TRANSACTION: Embedded namespace registration transaction.
        ACCOUNT_ADDRESS_RESTRICTION_TRANSACTION: Account address restriction transaction.
        EMBEDDED_ACCOUNT_ADDRESS_RESTRICTION_TRANSACTION: Embedded account address restriction transaction.
        ACCOUNT_MOSAIC_RESTRICTION_TRANSACTION: Account mosaic restriction transaction.
        EMBEDDED_ACCOUNT_MOSAIC_RESTRICTION_TRANSACTION: Embedded account mosaic restriction transaction.
        ACCOUNT_OPERATION_RESTRICTION_TRANSACTION: Account operation restriction transaction.
        EMBEDDED_ACCOUNT_OPERATION_RESTRICTION_TRANSACTION: Embedded account operation restriction transaction.
        MOSAIC_ADDRESS_RESTRICTION_TRANSACTION: Mosaic address restriction transaction.
        EMBEDDED_MOSAIC_ADDRESS_RESTRICTION_TRANSACTION: Embedded mosaic address restriction transaction.
        MOSAIC_GLOBAL_RESTRICTION_TRANSACTION: Mosaic global restriction transaction.
        EMBEDDED_MOSAIC_GLOBAL_RESTRICTION_TRANSACTION: Embedded mosaic global restriction transaction.
        TRANSFER_TRANSACTION: Transfer transaction.
        EMBEDDED_TRANSFER_TRANSACTION: Embedded transfer transaction.
    """

    RESERVED = 0
    NEMESIS_BLOCK_HEADER = 32835
    NORMAL_BLOCK_HEADER = 33091
    IMPORTANCE_BLOCK_HEADER = 33347
    ACCOUNT_KEY_LINK_TRANSACTION = 16716
    EMBEDDED_ACCOUNT_KEY_LINK_TRANSACTION = 16716
    NODE_KEY_LINK_TRANSACTION = 16972
    EMBEDDED_NODE_KEY_LINK_TRANSACTION = 16972
    AGGREGATE_COMPLETE_TRANSACTION = 16705
    AGGREGATE_BONDED_TRANSACTION = 16961
    VOTING_KEY_LINK_TRANSACTION = 16707
    EMBEDDED_VOTING_KEY_LINK_TRANSACTION = 16707
    VRF_KEY_LINK_TRANSACTION = 16963
    EMBEDDED_VRF_KEY_LINK_TRANSACTION = 16963
    HASH_LOCK_TRANSACTION = 16712
    EMBEDDED_HASH_LOCK_TRANSACTION = 16712
    SECRET_LOCK_TRANSACTION = 16722
    EMBEDDED_SECRET_LOCK_TRANSACTION = 16722
    SECRET_PROOF_TRANSACTION = 16978
    EMBEDDED_SECRET_PROOF_TRANSACTION = 16978
    ACCOUNT_METADATA_TRANSACTION = 16708
    EMBEDDED_ACCOUNT_METADATA_TRANSACTION = 16708
    MOSAIC_METADATA_TRANSACTION = 16964
    EMBEDDED_MOSAIC_METADATA_TRANSACTION = 16964
    NAMESPACE_METADATA_TRANSACTION = 17220
    EMBEDDED_NAMESPACE_METADATA_TRANSACTION = 17220
    MOSAIC_DEFINITION_TRANSACTION = 16717
    EMBEDDED_MOSAIC_DEFINITION_TRANSACTION = 16717
    MOSAIC_SUPPLY_CHANGE_TRANSACTION = 16973
    EMBEDDED_MOSAIC_SUPPLY_CHANGE_TRANSACTION = 16973
    MULTISIG_ACCOUNT_MODIFICATION_TRANSACTION = 16725
    EMBEDDED_MULTISIG_ACCOUNT_MODIFICATION_TRANSACTION = 16725
    ADDRESS_ALIAS_TRANSACTION = 16974
    EMBEDDED_ADDRESS_ALIAS_TRANSACTION = 16974
    MOSAIC_ALIAS_TRANSACTION = 17230
    EMBEDDED_MOSAIC_ALIAS_TRANSACTION = 17230
    NAMESPACE_REGISTRATION_TRANSACTION = 16718
    EMBEDDED_NAMESPACE_REGISTRATION_TRANSACTION = 16718
    ACCOUNT_ADDRESS_RESTRICTION_TRANSACTION = 16720
    EMBEDDED_ACCOUNT_ADDRESS_RESTRICTION_TRANSACTION = 16720
    ACCOUNT_MOSAIC_RESTRICTION_TRANSACTION = 16976
    EMBEDDED_ACCOUNT_MOSAIC_RESTRICTION_TRANSACTION = 16976
    ACCOUNT_OPERATION_RESTRICTION_TRANSACTION = 17232
    EMBEDDED_ACCOUNT_OPERATION_RESTRICTION_TRANSACTION = 17232
    MOSAIC_ADDRESS_RESTRICTION_TRANSACTION = 16977
    EMBEDDED_MOSAIC_ADDRESS_RESTRICTION_TRANSACTION = 16977
    MOSAIC_GLOBAL_RESTRICTION_TRANSACTION = 16721
    EMBEDDED_MOSAIC_GLOBAL_RESTRICTION_TRANSACTION = 16721
    TRANSFER_TRANSACTION = 16724
    EMBEDDED_TRANSFER_TRANSACTION = 16724

    @classmethod
    def load_from_binary(cls, payload: bytes) -> EntityTypeDto:
        """Creates an instance of EntityTypeDto from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of EntityTypeDto.
        """
        value: int = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes(payload), 2))
        return EntityTypeDto(value)

    @classmethod
    def get_size(cls) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        return 2

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.value, 2))
        return bytes_



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .MosaicBuilder import MosaicBuilder
from .ReceiptBuilder import ReceiptBuilder
from .ReceiptTypeDto import ReceiptTypeDto

class BalanceChangeReceiptBuilder(ReceiptBuilder):
    """Binary layout for a balance change receipt.

    Attributes:
        mosaic: Mosaic.
        targetAddress: Account address.
    """

    def __init__(self, version: int, type: ReceiptTypeDto, mosaic: MosaicBuilder, targetAddress: AddressDto):
        """Constructor.
        Args:
            version: Receipt version.
            type: Receipt type.
            mosaic: Mosaic.
            targetAddress: Account address.
        """
        super().__init__(version, type)
        self.mosaic = mosaic
        self.targetAddress = targetAddress


    @classmethod
    def load_from_binary(cls, payload: bytes) -> BalanceChangeReceiptBuilder:
        """Creates an instance of BalanceChangeReceiptBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of BalanceChangeReceiptBuilder.
        """
        bytes_ = bytes(payload)
        superObject = ReceiptBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        mosaic = MosaicBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[mosaic.get_size():]
        targetAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[targetAddress.get_size():]
        return BalanceChangeReceiptBuilder(superObject.version, superObject.type, mosaic, targetAddress)

    def get_mosaic(self) -> MosaicBuilder:
        """Gets mosaic.
        Returns:
            Mosaic.
        """
        return self.mosaic

    def get_target_address(self) -> AddressDto:
        """Gets account address.
        Returns:
            Account address.
        """
        return self.targetAddress

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.mosaic.get_size()
        size += self.targetAddress.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.mosaic.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.targetAddress.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .FinalizationEpochDto import FinalizationEpochDto
from .VotingKeyDto import VotingKeyDto

class PinnedVotingKeyBuilder:
    """Pinned voting key.

    Attributes:
        votingKey: Voting key.
        startEpoch: Start finalization epoch.
        endEpoch: End finalization epoch.
    """

    def __init__(self, votingKey: VotingKeyDto, startEpoch: FinalizationEpochDto, endEpoch: FinalizationEpochDto):
        """Constructor.
        Args:
            votingKey: Voting key.
            startEpoch: Start finalization epoch.
            endEpoch: End finalization epoch.
        """
        self.votingKey = votingKey
        self.startEpoch = startEpoch
        self.endEpoch = endEpoch


    @classmethod
    def load_from_binary(cls, payload: bytes) -> PinnedVotingKeyBuilder:
        """Creates an instance of PinnedVotingKeyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of PinnedVotingKeyBuilder.
        """
        bytes_ = bytes(payload)

        votingKey = VotingKeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[votingKey.get_size():]
        startEpoch = FinalizationEpochDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[startEpoch.get_size():]
        endEpoch = FinalizationEpochDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[endEpoch.get_size():]
        return PinnedVotingKeyBuilder(votingKey, startEpoch, endEpoch)

    def get_voting_key(self) -> VotingKeyDto:
        """Gets voting key.
        Returns:
            Voting key.
        """
        return self.votingKey

    def get_start_epoch(self) -> FinalizationEpochDto:
        """Gets start finalization epoch.
        Returns:
            Start finalization epoch.
        """
        return self.startEpoch

    def get_end_epoch(self) -> FinalizationEpochDto:
        """Gets end finalization epoch.
        Returns:
            End finalization epoch.
        """
        return self.endEpoch

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.votingKey.get_size()
        size += self.startEpoch.get_size()
        size += self.endEpoch.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.votingKey.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.startEpoch.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.endEpoch.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from enum import Enum
from .GeneratorUtils import GeneratorUtils


class MosaicRestrictionTypeDto(Enum):
    """Enumeration of mosaic restriction types

    Attributes:
        NONE: uninitialized value indicating no restriction.
        EQ: allow if equal.
        NE: allow if not equal.
        LT: allow if less than.
        LE: allow if less than or equal.
        GT: allow if greater than.
        GE: allow if greater than or equal.
    """

    NONE = 0
    EQ = 1
    NE = 2
    LT = 3
    LE = 4
    GT = 5
    GE = 6

    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicRestrictionTypeDto:
        """Creates an instance of MosaicRestrictionTypeDto from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicRestrictionTypeDto.
        """
        value: int = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes(payload), 1))
        return MosaicRestrictionTypeDto(value)

    @classmethod
    def get_size(cls) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        return 1

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.value, 1))
        return bytes_



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from binascii import hexlify
from .GeneratorUtils import GeneratorUtils

class ScopedMetadataKeyDto:
    """Scoped metadata key.

    Attributes:
        scopedMetadataKey: Scoped metadata key.
    """
    def __init__(self, scopedMetadataKey: int = 0):
        """Constructor.

        Args:
            scopedMetadataKey: Scoped metadata key.
        """
        self.scopedMetadataKey = scopedMetadataKey

    @classmethod
    def load_from_binary(cls, payload: bytes) -> ScopedMetadataKeyDto:
        """Creates an instance of ScopedMetadataKeyDto from binary payload.

        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of ScopedMetadataKeyDto.
        """
        bytes_ = bytes(payload)
        scopedMetadataKey = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))
        return ScopedMetadataKeyDto(scopedMetadataKey)

    @classmethod
    def get_size(cls) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        return 8

    def get_scoped_metadata_key(self) -> int:
        """Gets Scoped metadata key.

        Returns:
            Scoped metadata key.
        """
        return self.scopedMetadataKey

    def serialize(self) -> bytes:
        """Serializes self to bytes.

        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_scoped_metadata_key(), 8))
        return bytes_

    def __str__(self):
        result = hexlify(GeneratorUtils.uint_to_buffer(self.get_scoped_metadata_key(), 8)).decode('utf-8')
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

# pylint: disable=R0911,R0912

# Imports for creating embedded transaction builders
from .EmbeddedTransactionBuilder import EmbeddedTransactionBuilder
from .EmbeddedAccountAddressRestrictionTransactionBuilder import EmbeddedAccountAddressRestrictionTransactionBuilder
from .EmbeddedAccountKeyLinkTransactionBuilder import EmbeddedAccountKeyLinkTransactionBuilder
from .EmbeddedAccountMetadataTransactionBuilder import EmbeddedAccountMetadataTransactionBuilder
from .EmbeddedAccountMosaicRestrictionTransactionBuilder import EmbeddedAccountMosaicRestrictionTransactionBuilder
from .EmbeddedAccountOperationRestrictionTransactionBuilder import EmbeddedAccountOperationRestrictionTransactionBuilder
from .EmbeddedAddressAliasTransactionBuilder import EmbeddedAddressAliasTransactionBuilder
from .EmbeddedHashLockTransactionBuilder import EmbeddedHashLockTransactionBuilder
from .EmbeddedMosaicAddressRestrictionTransactionBuilder import EmbeddedMosaicAddressRestrictionTransactionBuilder
from .EmbeddedMosaicAliasTransactionBuilder import EmbeddedMosaicAliasTransactionBuilder
from .EmbeddedMosaicDefinitionTransactionBuilder import EmbeddedMosaicDefinitionTransactionBuilder
from .EmbeddedMosaicGlobalRestrictionTransactionBuilder import EmbeddedMosaicGlobalRestrictionTransactionBuilder
from .EmbeddedMosaicMetadataTransactionBuilder import EmbeddedMosaicMetadataTransactionBuilder
from .EmbeddedMosaicSupplyChangeTransactionBuilder import EmbeddedMosaicSupplyChangeTransactionBuilder
from .EmbeddedMultisigAccountModificationTransactionBuilder import EmbeddedMultisigAccountModificationTransactionBuilder
from .EmbeddedNamespaceMetadataTransactionBuilder import EmbeddedNamespaceMetadataTransactionBuilder
from .EmbeddedNamespaceRegistrationTransactionBuilder import EmbeddedNamespaceRegistrationTransactionBuilder
from .EmbeddedNodeKeyLinkTransactionBuilder import EmbeddedNodeKeyLinkTransactionBuilder
from .EmbeddedSecretLockTransactionBuilder import EmbeddedSecretLockTransactionBuilder
from .EmbeddedSecretProofTransactionBuilder import EmbeddedSecretProofTransactionBuilder
from .EmbeddedTransferTransactionBuilder import EmbeddedTransferTransactionBuilder
from .EmbeddedVotingKeyLinkTransactionBuilder import EmbeddedVotingKeyLinkTransactionBuilder
from .EmbeddedVrfKeyLinkTransactionBuilder import EmbeddedVrfKeyLinkTransactionBuilder

class EmbeddedTransactionBuilderFactory:
    """Factory in charge of creating the specific embedded transaction builder from the binary payload.
    """

    @classmethod
    def create_from_payload(cls, payload) -> EmbeddedTransactionBuilder:
        """
        It creates the specific embedded transaction builder from the payload bytes.
        Args:
            payload: bytes
        Returns:
            the EmbeddedTransactionBuilder subclass
        """
        headerBuilder = EmbeddedTransactionBuilder.load_from_binary(payload)
        entityType = headerBuilder.type
        entityTypeVersion = headerBuilder.version
        if entityType == 0x414c and entityTypeVersion == 1:
            return EmbeddedAccountKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x424c and entityTypeVersion == 1:
            return EmbeddedNodeKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4143 and entityTypeVersion == 1:
            return EmbeddedVotingKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4243 and entityTypeVersion == 1:
            return EmbeddedVrfKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4148 and entityTypeVersion == 1:
            return EmbeddedHashLockTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4152 and entityTypeVersion == 1:
            return EmbeddedSecretLockTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4252 and entityTypeVersion == 1:
            return EmbeddedSecretProofTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4144 and entityTypeVersion == 1:
            return EmbeddedAccountMetadataTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4244 and entityTypeVersion == 1:
            return EmbeddedMosaicMetadataTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4344 and entityTypeVersion == 1:
            return EmbeddedNamespaceMetadataTransactionBuilder.load_from_binary(payload)
        if entityType == 0x414d and entityTypeVersion == 1:
            return EmbeddedMosaicDefinitionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x424d and entityTypeVersion == 1:
            return EmbeddedMosaicSupplyChangeTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4155 and entityTypeVersion == 1:
            return EmbeddedMultisigAccountModificationTransactionBuilder.load_from_binary(payload)
        if entityType == 0x424e and entityTypeVersion == 1:
            return EmbeddedAddressAliasTransactionBuilder.load_from_binary(payload)
        if entityType == 0x434e and entityTypeVersion == 1:
            return EmbeddedMosaicAliasTransactionBuilder.load_from_binary(payload)
        if entityType == 0x414e and entityTypeVersion == 1:
            return EmbeddedNamespaceRegistrationTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4150 and entityTypeVersion == 1:
            return EmbeddedAccountAddressRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4250 and entityTypeVersion == 1:
            return EmbeddedAccountMosaicRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4350 and entityTypeVersion == 1:
            return EmbeddedAccountOperationRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4251 and entityTypeVersion == 1:
            return EmbeddedMosaicAddressRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4151 and entityTypeVersion == 1:
            return EmbeddedMosaicGlobalRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4154 and entityTypeVersion == 1:
            return EmbeddedTransferTransactionBuilder.load_from_binary(payload)
        return headerBuilder

    @classmethod
    def create_by_name(cls, transaction_name, signer_public_key, network) -> EmbeddedTransactionBuilder:
        """
        It creates the specific embedded transaction builder given name, network and signer.
        Args:
            transaction_name: transaction name
            signer_public_key: signer
            network: network
        Returns:
            the EmbeddedTransactionBuilder subclass
        """
        mapping = {
            'accountKeyLink': EmbeddedAccountKeyLinkTransactionBuilder,
            'nodeKeyLink': EmbeddedNodeKeyLinkTransactionBuilder,
            'votingKeyLink': EmbeddedVotingKeyLinkTransactionBuilder,
            'vrfKeyLink': EmbeddedVrfKeyLinkTransactionBuilder,
            'hashLock': EmbeddedHashLockTransactionBuilder,
            'secretLock': EmbeddedSecretLockTransactionBuilder,
            'secretProof': EmbeddedSecretProofTransactionBuilder,
            'accountMetadata': EmbeddedAccountMetadataTransactionBuilder,
            'mosaicMetadata': EmbeddedMosaicMetadataTransactionBuilder,
            'namespaceMetadata': EmbeddedNamespaceMetadataTransactionBuilder,
            'mosaicDefinition': EmbeddedMosaicDefinitionTransactionBuilder,
            'mosaicSupplyChange': EmbeddedMosaicSupplyChangeTransactionBuilder,
            'multisigAccountModification': EmbeddedMultisigAccountModificationTransactionBuilder,
            'addressAlias': EmbeddedAddressAliasTransactionBuilder,
            'mosaicAlias': EmbeddedMosaicAliasTransactionBuilder,
            'namespaceRegistration': EmbeddedNamespaceRegistrationTransactionBuilder,
            'accountAddressRestriction': EmbeddedAccountAddressRestrictionTransactionBuilder,
            'accountMosaicRestriction': EmbeddedAccountMosaicRestrictionTransactionBuilder,
            'accountOperationRestriction': EmbeddedAccountOperationRestrictionTransactionBuilder,
            'mosaicAddressRestriction': EmbeddedMosaicAddressRestrictionTransactionBuilder,
            'mosaicGlobalRestriction': EmbeddedMosaicGlobalRestrictionTransactionBuilder,
            'transfer': EmbeddedTransferTransactionBuilder,
        }
        if transaction_name not in mapping:
            raise ValueError('transaction named {} is not supported'.format(transaction_name))

        return mapping[transaction_name](signer_public_key, network)



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AmountDto import AmountDto
from .ImportanceHeightDto import ImportanceHeightDto

class HeightActivityBucketBuilder:
    """Account activity bucket.

    Attributes:
        startHeight: Activity start height.
        totalFeesPaid: Total fees paid by account.
        beneficiaryCount: Number of times account has been used as a beneficiary.
        rawScore: Raw importance score.
    """

    def __init__(self, startHeight: ImportanceHeightDto, totalFeesPaid: AmountDto, beneficiaryCount: int, rawScore: int):
        """Constructor.
        Args:
            startHeight: Activity start height.
            totalFeesPaid: Total fees paid by account.
            beneficiaryCount: Number of times account has been used as a beneficiary.
            rawScore: Raw importance score.
        """
        self.startHeight = startHeight
        self.totalFeesPaid = totalFeesPaid
        self.beneficiaryCount = beneficiaryCount
        self.rawScore = rawScore


    @classmethod
    def load_from_binary(cls, payload: bytes) -> HeightActivityBucketBuilder:
        """Creates an instance of HeightActivityBucketBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of HeightActivityBucketBuilder.
        """
        bytes_ = bytes(payload)

        startHeight = ImportanceHeightDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[startHeight.get_size():]
        totalFeesPaid = AmountDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[totalFeesPaid.get_size():]
        beneficiaryCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        rawScore = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        return HeightActivityBucketBuilder(startHeight, totalFeesPaid, beneficiaryCount, rawScore)

    def get_start_height(self) -> ImportanceHeightDto:
        """Gets activity start height.
        Returns:
            Activity start height.
        """
        return self.startHeight

    def get_total_fees_paid(self) -> AmountDto:
        """Gets total fees paid by account.
        Returns:
            Total fees paid by account.
        """
        return self.totalFeesPaid

    def get_beneficiary_count(self) -> int:
        """Gets number of times account has been used as a beneficiary.
        Returns:
            Number of times account has been used as a beneficiary.
        """
        return self.beneficiaryCount

    def get_raw_score(self) -> int:
        """Gets raw importance score.
        Returns:
            Raw importance score.
        """
        return self.rawScore

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.startHeight.get_size()
        size += self.totalFeesPaid.get_size()
        size += 4  # beneficiaryCount
        size += 8  # rawScore
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.startHeight.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.totalFeesPaid.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_beneficiary_count(), 4))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_raw_score(), 8))  # kind:SIMPLE
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .BlockDurationDto import BlockDurationDto
from .Hash256Dto import Hash256Dto
from .UnresolvedMosaicBuilder import UnresolvedMosaicBuilder

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class HashLockTransactionBodyBuilder:
    """Binary layout for a hash lock transaction.

    Attributes:
        mosaic: Lock mosaic.
        duration: Number of blocks for which a lock should be valid.
        hash: Lock hash.
    """
    def __init__(self):
        """ Constructor."""
        self.mosaic = None # UnresolvedMosaicBuilder
        self.duration = BlockDurationDto().blockDuration
        self.hash = bytes(32)

    @classmethod
    def load_from_binary(cls, payload: bytes) -> HashLockTransactionBodyBuilder:
        """Creates an instance of HashLockTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of HashLockTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        mosaic = UnresolvedMosaicBuilder.load_from_binary(bytes_)  # kind:CUSTOM1_nonbyte
        bytes_ = bytes_[mosaic.get_size():]
        mosaic = mosaic.as_tuple()
        duration_ = BlockDurationDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        duration = duration_.blockDuration
        bytes_ = bytes_[duration_.get_size():]
        hash_ = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        hash = hash_.hash256
        bytes_ = bytes_[hash_.get_size():]

        # create object and call
        result = HashLockTransactionBodyBuilder()
        result.mosaic = mosaic
        result.duration = duration
        result.hash = hash
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += UnresolvedMosaicBuilder.from_tuple(self.mosaic).get_size()
        size += BlockDurationDto(self.duration).get_size()
        size += Hash256Dto(self.hash).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedMosaicBuilder.from_tuple(self.mosaic).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, BlockDurationDto(self.duration).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, Hash256Dto(self.hash).serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('mosaic', to_hex_string(UnresolvedMosaicBuilder.from_tuple(self.mosaic).serialize()))
        result += '{:24s} : {}\n'.format('duration', to_hex_string(BlockDurationDto(self.duration).serialize()))
        result += '{:24s} : {}\n'.format('hash', to_hex_string(Hash256Dto(self.hash).serialize()))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .MetadataTypeDto import MetadataTypeDto
from .MetadataValueBuilder import MetadataValueBuilder
from .ScopedMetadataKeyDto import ScopedMetadataKeyDto
from .StateHeaderBuilder import StateHeaderBuilder

class MetadataEntryBuilder(StateHeaderBuilder):
    """Binary layout of a metadata entry.

    Attributes:
        sourceAddress: Metadata source address (provider).
        targetAddress: Metadata target address.
        scopedMetadataKey: Metadata key scoped to source, target and type.
        targetId: Target id.
        metadataType: Metadata type.
        value: Value.
    """

    def __init__(self, version: int, sourceAddress: AddressDto, targetAddress: AddressDto, scopedMetadataKey: ScopedMetadataKeyDto, targetId: int, metadataType: MetadataTypeDto, value: MetadataValueBuilder):
        """Constructor.
        Args:
            version: Serialization version.
            sourceAddress: Metadata source address (provider).
            targetAddress: Metadata target address.
            scopedMetadataKey: Metadata key scoped to source, target and type.
            targetId: Target id.
            metadataType: Metadata type.
            value: Value.
        """
        super().__init__(version)
        self.sourceAddress = sourceAddress
        self.targetAddress = targetAddress
        self.scopedMetadataKey = scopedMetadataKey
        self.targetId = targetId
        self.metadataType = metadataType
        self.value = value


    @classmethod
    def load_from_binary(cls, payload: bytes) -> MetadataEntryBuilder:
        """Creates an instance of MetadataEntryBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MetadataEntryBuilder.
        """
        bytes_ = bytes(payload)
        superObject = StateHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        sourceAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[sourceAddress.get_size():]
        targetAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[targetAddress.get_size():]
        scopedMetadataKey = ScopedMetadataKeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[scopedMetadataKey.get_size():]
        targetId = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        metadataType = MetadataTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[metadataType.get_size():]
        value = MetadataValueBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[value.get_size():]
        return MetadataEntryBuilder(superObject.version, sourceAddress, targetAddress, scopedMetadataKey, targetId, metadataType, value)

    def get_source_address(self) -> AddressDto:
        """Gets metadata source address (provider).
        Returns:
            Metadata source address (provider).
        """
        return self.sourceAddress

    def get_target_address(self) -> AddressDto:
        """Gets metadata target address.
        Returns:
            Metadata target address.
        """
        return self.targetAddress

    def get_scoped_metadata_key(self) -> ScopedMetadataKeyDto:
        """Gets metadata key scoped to source, target and type.
        Returns:
            Metadata key scoped to source, target and type.
        """
        return self.scopedMetadataKey

    def get_target_id(self) -> int:
        """Gets target id.
        Returns:
            Target id.
        """
        return self.targetId

    def get_metadata_type(self) -> MetadataTypeDto:
        """Gets metadata type.
        Returns:
            Metadata type.
        """
        return self.metadataType

    def get_value(self) -> MetadataValueBuilder:
        """Gets value.
        Returns:
            Value.
        """
        return self.value

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.sourceAddress.get_size()
        size += self.targetAddress.get_size()
        size += self.scopedMetadataKey.get_size()
        size += 8  # targetId
        size += self.metadataType.get_size()
        size += self.value.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.sourceAddress.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.targetAddress.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.scopedMetadataKey.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_target_id(), 8))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.metadataType.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.value.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .MosaicAddressRestrictionEntryBuilder import MosaicAddressRestrictionEntryBuilder
from .MosaicGlobalRestrictionEntryBuilder import MosaicGlobalRestrictionEntryBuilder
from .MosaicRestrictionEntryTypeDto import MosaicRestrictionEntryTypeDto
from .StateHeaderBuilder import StateHeaderBuilder

class MosaicRestrictionEntryBuilder(StateHeaderBuilder):
    """Binary layout for a mosaic restriction.

    Attributes:
        entryType: Type of restriction being placed upon the entity.
        addressEntry: Address restriction rule.
        globalEntry: Global mosaic rule.
    """

    def __init__(self, version: int, entryType: MosaicRestrictionEntryTypeDto, addressEntry: MosaicAddressRestrictionEntryBuilder, globalEntry: MosaicGlobalRestrictionEntryBuilder):
        """Constructor.
        Args:
            version: Serialization version.
            entryType: Type of restriction being placed upon the entity.
            addressEntry: Address restriction rule.
            globalEntry: Global mosaic rule.
        """
        super().__init__(version)
        self.entryType = entryType
        self.addressEntry = addressEntry
        self.globalEntry = globalEntry


    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicRestrictionEntryBuilder:
        """Creates an instance of MosaicRestrictionEntryBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicRestrictionEntryBuilder.
        """
        bytes_ = bytes(payload)
        superObject = StateHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        entryType = MosaicRestrictionEntryTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[entryType.get_size():]
        addressEntry = None
        if entryType == MosaicRestrictionEntryTypeDto.ADDRESS:
            addressEntry = MosaicAddressRestrictionEntryBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[addressEntry.get_size():]
        globalEntry = None
        if entryType == MosaicRestrictionEntryTypeDto.GLOBAL:
            globalEntry = MosaicGlobalRestrictionEntryBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[globalEntry.get_size():]
        return MosaicRestrictionEntryBuilder(superObject.version, entryType, addressEntry, globalEntry)

    def get_entry_type(self) -> MosaicRestrictionEntryTypeDto:
        """Gets type of restriction being placed upon the entity.
        Returns:
            Type of restriction being placed upon the entity.
        """
        return self.entryType

    def get_address_entry(self) -> MosaicAddressRestrictionEntryBuilder:
        """Gets address restriction rule.
        Returns:
            Address restriction rule.
        """
        if not self.entryType == MosaicRestrictionEntryTypeDto.ADDRESS:
            raise Exception('entryType is not set to ADDRESS.')
        return self.addressEntry

    def get_global_entry(self) -> MosaicGlobalRestrictionEntryBuilder:
        """Gets global mosaic rule.
        Returns:
            Global mosaic rule.
        """
        if not self.entryType == MosaicRestrictionEntryTypeDto.GLOBAL:
            raise Exception('entryType is not set to GLOBAL.')
        return self.globalEntry

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.entryType.get_size()
        if self.entryType == MosaicRestrictionEntryTypeDto.ADDRESS:
            size += self.addressEntry.get_size()
        if self.entryType == MosaicRestrictionEntryTypeDto.GLOBAL:
            size += self.globalEntry.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.entryType.serialize())  # kind:CUSTOM
        if self.entryType == MosaicRestrictionEntryTypeDto.ADDRESS:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.addressEntry.serialize())  # kind:CUSTOM
        if self.entryType == MosaicRestrictionEntryTypeDto.GLOBAL:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.globalEntry.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List, TypeVar

T = TypeVar('T')

class GeneratorUtils:
    """Generator utility class"""

    @staticmethod
    def buffer_to_uint(buffer: bytes) -> int:
        return int.from_bytes(buffer, byteorder='little', signed=False)

    @staticmethod
    def uint_to_buffer(uint: int, buffer_size: int) -> bytes:
        return uint.to_bytes(buffer_size, byteorder='little', signed=False)

    @staticmethod
    def concat_typed_arrays(array1, array2):
        return array1 + array2

    @staticmethod
    def get_transaction_padding_size(size: int, alignment: int) -> int:
        if size % alignment == 0:
            return 0
        return alignment - (size % alignment)

    @staticmethod
    def get_bytes(binary: bytes, size: int) -> bytes:
        if size > len(binary):
            raise Exception('size should not exceed {0}. The value of size was: {1}'.format(len(binary), size))
        return binary[0:size]

    # pylint: disable=bad-staticmethod-argument
    # cls argument is not GeneratorUtils
    @staticmethod
    def load_from_binary(cls: T, items: List[cls], payload: bytes, payloadSize: int):
        remainingByteSizes = payloadSize
        while remainingByteSizes > 0:
            item = cls.load_from_binary(payload)
            items.append(item)
            itemSize = item.get_size()
            remainingByteSizes -= itemSize
            payload = payload[itemSize:]
        return payload



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .StateHeaderBuilder import StateHeaderBuilder

class MultisigEntryBuilder(StateHeaderBuilder):
    """Binary layout for a multisig entry.

    Attributes:
        minApproval: Minimum approval for modifications.
        minRemoval: Minimum approval for removal.
        accountAddress: Account address.
        cosignatoryAddresses: Cosignatories for account.
        multisigAddresses: Accounts for which the entry is cosignatory.
    """

    def __init__(self, version: int, minApproval: int, minRemoval: int, accountAddress: AddressDto, cosignatoryAddresses: List[AddressDto], multisigAddresses: List[AddressDto]):
        """Constructor.
        Args:
            version: Serialization version.
            minApproval: Minimum approval for modifications.
            minRemoval: Minimum approval for removal.
            accountAddress: Account address.
            cosignatoryAddresses: Cosignatories for account.
            multisigAddresses: Accounts for which the entry is cosignatory.
        """
        super().__init__(version)
        self.minApproval = minApproval
        self.minRemoval = minRemoval
        self.accountAddress = accountAddress
        self.cosignatoryAddresses = cosignatoryAddresses
        self.multisigAddresses = multisigAddresses


    @classmethod
    def load_from_binary(cls, payload: bytes) -> MultisigEntryBuilder:
        """Creates an instance of MultisigEntryBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MultisigEntryBuilder.
        """
        bytes_ = bytes(payload)
        superObject = StateHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        minApproval = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        minRemoval = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        accountAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[accountAddress.get_size():]
        cosignatoryAddressesCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIZE_FIELD
        bytes_ = bytes_[8:]
        cosignatoryAddresses: List[AddressDto] = []  # kind:ARRAY
        for _ in range(cosignatoryAddressesCount):
            item = AddressDto.load_from_binary(bytes_)
            cosignatoryAddresses.append(item)
            bytes_ = bytes_[item.get_size():]
        multisigAddressesCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIZE_FIELD
        bytes_ = bytes_[8:]
        multisigAddresses: List[AddressDto] = []  # kind:ARRAY
        for _ in range(multisigAddressesCount):
            item = AddressDto.load_from_binary(bytes_)
            multisigAddresses.append(item)
            bytes_ = bytes_[item.get_size():]
        return MultisigEntryBuilder(superObject.version, minApproval, minRemoval, accountAddress, cosignatoryAddresses, multisigAddresses)

    def get_min_approval(self) -> int:
        """Gets minimum approval for modifications.
        Returns:
            Minimum approval for modifications.
        """
        return self.minApproval

    def get_min_removal(self) -> int:
        """Gets minimum approval for removal.
        Returns:
            Minimum approval for removal.
        """
        return self.minRemoval

    def get_account_address(self) -> AddressDto:
        """Gets account address.
        Returns:
            Account address.
        """
        return self.accountAddress

    def get_cosignatory_addresses(self) -> List[AddressDto]:
        """Gets cosignatories for account.
        Returns:
            Cosignatories for account.
        """
        return self.cosignatoryAddresses

    def get_multisig_addresses(self) -> List[AddressDto]:
        """Gets accounts for which the entry is cosignatory.
        Returns:
            Accounts for which the entry is cosignatory.
        """
        return self.multisigAddresses

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += 4  # minApproval
        size += 4  # minRemoval
        size += self.accountAddress.get_size()
        size += 8  # cosignatoryAddressesCount
        for _ in self.cosignatoryAddresses:
            size += _.get_size()
        size += 8  # multisigAddressesCount
        for _ in self.multisigAddresses:
            size += _.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_min_approval(), 4))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_min_removal(), 4))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.accountAddress.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(len(self.get_cosignatory_addresses()), 8))  # kind:SIZE_FIELD
        for _ in self.cosignatoryAddresses: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(len(self.get_multisig_addresses()), 8))  # kind:SIZE_FIELD
        for _ in self.multisigAddresses: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .AmountDto import AmountDto
from .EntityTypeDto import EntityTypeDto
from .KeyDto import KeyDto
from .NetworkTypeDto import NetworkTypeDto
from .SignatureDto import SignatureDto
from .TimestampDto import TimestampDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class TransactionBuilder:
    """Binary layout for a transaction.

    Attributes:
        signature: Entity signature.
        signer_public_key: Entity signer's public key.
        version: Entity version.
        network: Entity network.
        type: Entity type.
        fee: Transaction fee.
        deadline: Transaction deadline.
    """
    type_hints = {
        'signature' : 'SignatureDto',
        'signer_public_key' : 'KeyDto',
        'network' : 'NetworkTypeDto',
        'type' : 'EntityTypeDto',
        'fee' : 'AmountDto',
        'deadline' : 'TimestampDto',
    }

    def __init__(self, signer_public_key, version, network, type):
        """Constructor.
        Args:
            signer_public_key: Entity signer's public key.
            version: Entity version.
            network: Entity network.
            type: Entity type.
        """
        self.signature = bytes(64)
        self.signer_public_key = signer_public_key
        self.version = version
        self.network = network
        self.type = type

        self.fee = 0
        self.deadline = 0


    @classmethod
    def load_from_binary(cls, payload: bytes) -> TransactionBuilder:
        """Creates an instance of TransactionBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of TransactionBuilder.
        """
        bytes_ = bytes(payload)

        size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        verifiable_entity_header__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        signature_ = SignatureDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        signature = signature_.signature
        bytes_ = bytes_[signature_.get_size():]
        signer_public_key_ = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        signer_public_key = signer_public_key_.key
        bytes_ = bytes_[signer_public_key_.get_size():]
        entity_body__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        version = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]
        network_ = NetworkTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        network = network_.value
        bytes_ = bytes_[network_.get_size():]
        type_ = EntityTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        type = type_.value
        bytes_ = bytes_[type_.get_size():]
        fee_ = AmountDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        fee = fee_.amount
        bytes_ = bytes_[fee_.get_size():]
        deadline_ = TimestampDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        deadline = deadline_.timestamp
        bytes_ = bytes_[deadline_.get_size():]

        # create object and call
        result = TransactionBuilder(signer_public_key, version, network, type)
        result.signature = signature
        result.fee = fee
        result.deadline = deadline
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 4  # size
        size += 4  # verifiable_entity_header__reserved1
        size += SignatureDto(self.signature).get_size()
        size += KeyDto(self.signer_public_key).get_size()
        size += 4  # entity_body__reserved1
        size += 1  # version
        size += NetworkTypeDto(self.network).get_size()
        size += EntityTypeDto(self.type).get_size()
        size += AmountDto(self.fee).get_size()
        size += TimestampDto(self.deadline).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_size(), 4))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, SignatureDto(self.signature).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, KeyDto(self.signer_public_key).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.version, 1))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, NetworkTypeDto(self.network).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, EntityTypeDto(self.type).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, AmountDto(self.fee).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, TimestampDto(self.deadline).serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('size', to_hex_string(GeneratorUtils.uint_to_buffer(self.get_size(), 4)))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        result += '{:24s} : {}\n'.format('signature', to_hex_string(SignatureDto(self.signature).serialize()))
        result += '{:24s} : {}\n'.format('signer_public_key', to_hex_string(KeyDto(self.signer_public_key).serialize()))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        result += '{:24s} : {}\n'.format('version', to_hex_string(GeneratorUtils.uint_to_buffer(self.version, 1)))
        result += '{:24s} : {}\n'.format('network', to_hex_string(NetworkTypeDto(self.network).serialize()))
        result += '{:24s} : {}\n'.format('type', to_hex_string(EntityTypeDto(self.type).serialize()))
        result += '{:24s} : {}\n'.format('fee', to_hex_string(AmountDto(self.fee).serialize()))
        result += '{:24s} : {}\n'.format('deadline', to_hex_string(TimestampDto(self.deadline).serialize()))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .BlockDurationDto import BlockDurationDto
from .NamespaceIdDto import NamespaceIdDto
from .NamespaceRegistrationTypeDto import NamespaceRegistrationTypeDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class NamespaceRegistrationTransactionBodyBuilder:
    """Binary layout for a namespace registration transaction.

    Attributes:
        duration: Namespace duration.
        parent_id: Parent namespace identifier.
        id: Namespace identifier.
        registration_type: Namespace registration type.
        name: Namespace name.
    """
    def __init__(self):
        """ Constructor."""
        self.duration = BlockDurationDto().blockDuration
        self.parent_id = NamespaceIdDto().namespaceId
        self.id = NamespaceIdDto().namespaceId
        self.registration_type = NamespaceRegistrationTypeDto(0).value
        self.name = bytes()

    @classmethod
    def load_from_binary(cls, payload: bytes) -> NamespaceRegistrationTransactionBodyBuilder:
        """Creates an instance of NamespaceRegistrationTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of NamespaceRegistrationTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)
        registrationTypeCondition = bytes_[0:8]
        bytes_ = bytes_[8:]

        id_ = NamespaceIdDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        id = id_.namespaceId
        bytes_ = bytes_[id_.get_size():]
        registration_type_ = NamespaceRegistrationTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        registration_type = registration_type_.value
        bytes_ = bytes_[registration_type_.get_size():]
        name_size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIZE_FIELD
        bytes_ = bytes_[1:]
        name = GeneratorUtils.get_bytes(bytes_, name_size)  # kind:BUFFER
        bytes_ = bytes_[name_size:]
        duration = None
        if registration_type == NamespaceRegistrationTypeDto.ROOT.value:
            duration = BlockDurationDto.load_from_binary(registrationTypeCondition).blockDuration  # kind:CUSTOM3
        parent_id = None
        if registration_type == NamespaceRegistrationTypeDto.CHILD.value:
            parent_id = NamespaceIdDto.load_from_binary(registrationTypeCondition).namespaceId  # kind:CUSTOM3

        # create object and call
        result = NamespaceRegistrationTransactionBodyBuilder()
        result.duration = duration
        result.parent_id = parent_id
        result.id = id
        result.registration_type = registration_type
        result.name = name
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        if self.registration_type == NamespaceRegistrationTypeDto.ROOT.value:
            size += BlockDurationDto(self.duration).get_size()
        if self.registration_type == NamespaceRegistrationTypeDto.CHILD.value:
            size += NamespaceIdDto(self.parent_id).get_size()
        size += NamespaceIdDto(self.id).get_size()
        size += NamespaceRegistrationTypeDto(self.registration_type).get_size()
        size += 1  # name_size
        size += len(self.name)
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        if self.registration_type == NamespaceRegistrationTypeDto.ROOT.value:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, BlockDurationDto(self.duration).serialize())  # kind:CUSTOM
        if self.registration_type == NamespaceRegistrationTypeDto.CHILD.value:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, NamespaceIdDto(self.parent_id).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, NamespaceIdDto(self.id).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, NamespaceRegistrationTypeDto(self.registration_type).serialize())  # kind:CUSTOM
        size_value = len(self.name)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 1))  # kind:SIZE_FIELD
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.name)  # kind:BUFFER
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        if self.registration_type == NamespaceRegistrationTypeDto.ROOT.value:
            result += '{:24s} : {}\n'.format('duration', to_hex_string(BlockDurationDto(self.duration).serialize()))
        if self.registration_type == NamespaceRegistrationTypeDto.CHILD.value:
            result += '{:24s} : {}\n'.format('parent_id', to_hex_string(NamespaceIdDto(self.parent_id).serialize()))
        result += '{:24s} : {}\n'.format('id', to_hex_string(NamespaceIdDto(self.id).serialize()))
        result += '{:24s} : {}\n'.format('registration_type', to_hex_string(NamespaceRegistrationTypeDto(self.registration_type).serialize()))
        size_value = len(self.name)
        result += '{:24s} : {}\n'.format('name_size', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 1)))
        result += '{:24s} : {}\n'.format('name', to_hex_string(self.name))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .AmountDto import AmountDto
from .MosaicSupplyChangeActionDto import MosaicSupplyChangeActionDto
from .UnresolvedMosaicIdDto import UnresolvedMosaicIdDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class MosaicSupplyChangeTransactionBodyBuilder:
    """Binary layout for a mosaic supply change transaction.

    Attributes:
        mosaic_id: Affected mosaic identifier.
        delta: Change amount.
        action: Supply change action.
    """
    def __init__(self):
        """ Constructor."""
        self.mosaic_id = UnresolvedMosaicIdDto().unresolvedMosaicId
        self.delta = AmountDto().amount
        self.action = MosaicSupplyChangeActionDto(0).value

    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicSupplyChangeTransactionBodyBuilder:
        """Creates an instance of MosaicSupplyChangeTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicSupplyChangeTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        mosaic_id_ = UnresolvedMosaicIdDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        mosaic_id = mosaic_id_.unresolvedMosaicId
        bytes_ = bytes_[mosaic_id_.get_size():]
        delta_ = AmountDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        delta = delta_.amount
        bytes_ = bytes_[delta_.get_size():]
        action_ = MosaicSupplyChangeActionDto.load_from_binary(bytes_)  # kind:CUSTOM2
        action = action_.value
        bytes_ = bytes_[action_.get_size():]

        # create object and call
        result = MosaicSupplyChangeTransactionBodyBuilder()
        result.mosaic_id = mosaic_id
        result.delta = delta
        result.action = action
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += UnresolvedMosaicIdDto(self.mosaic_id).get_size()
        size += AmountDto(self.delta).get_size()
        size += MosaicSupplyChangeActionDto(self.action).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedMosaicIdDto(self.mosaic_id).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, AmountDto(self.delta).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, MosaicSupplyChangeActionDto(self.action).serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('mosaic_id', to_hex_string(UnresolvedMosaicIdDto(self.mosaic_id).serialize()))
        result += '{:24s} : {}\n'.format('delta', to_hex_string(AmountDto(self.delta).serialize()))
        result += '{:24s} : {}\n'.format('action', to_hex_string(MosaicSupplyChangeActionDto(self.action).serialize()))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .HeightActivityBucketBuilder import HeightActivityBucketBuilder

class HeightActivityBucketsBuilder:
    """Account activity buckets.

    Attributes:
        buckets: Account activity buckets.
    """

    def __init__(self, buckets: List[HeightActivityBucketBuilder]):
        """Constructor.
        Args:
            buckets: Account activity buckets.
        """
        self.buckets = buckets


    @classmethod
    def load_from_binary(cls, payload: bytes) -> HeightActivityBucketsBuilder:
        """Creates an instance of HeightActivityBucketsBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of HeightActivityBucketsBuilder.
        """
        bytes_ = bytes(payload)

        buckets: List[HeightActivityBucketBuilder] = []  # kind:ARRAY
        for _ in range(5):
            item = HeightActivityBucketBuilder.load_from_binary(bytes_)
            buckets.append(item)
            bytes_ = bytes_[item.get_size():]
        return HeightActivityBucketsBuilder(buckets)

    def get_buckets(self) -> List[HeightActivityBucketBuilder]:
        """Gets account activity buckets.
        Returns:
            Account activity buckets.
        """
        return self.buckets

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        for _ in self.buckets:
            size += _.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        for _ in self.buckets: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from enum import Flag
from typing import List
from .GeneratorUtils import GeneratorUtils


class AccountKeyTypeFlagsDto(Flag):
    """Enumeration of account key type flags

    Attributes:
        UNSET: unset key.
        LINKED: linked account public key \note this can be either a remote or main account public key depending on context.
        NODE: node public key on which remote is allowed to harvest.
        VRF: VRF public key.
    """

    UNSET = 0
    LINKED = 1
    NODE = 2
    VRF = 4

    @classmethod
    def load_from_binary(cls, payload: bytes) -> AccountKeyTypeFlagsDto:
        """Creates an instance of AccountKeyTypeFlagsDto from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AccountKeyTypeFlagsDto.
        """
        value: int = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes(payload), 1))
        return AccountKeyTypeFlagsDto(value)

    @classmethod
    def get_size(cls) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        return 1

    @classmethod
    def bytesToFlags(cls, bitMaskValue: bytes, size: int) -> List[AccountKeyTypeFlagsDto]:
        """Converts a bit representation to a list of AccountKeyTypeFlagsDto.
        Args:
            bitMaskValue Bitmask bytes value.
        Returns:
            List of AccountKeyTypeFlagsDto flags representing the int value.
        """
        return cls.intToFlags(GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bitMaskValue, size)))

    @classmethod
    def intToFlags(cls, bitMaskValue: int) -> List[AccountKeyTypeFlagsDto]:
        """Converts a bit representation to a list of AccountKeyTypeFlagsDto.
        Args:
            bitMaskValue Bitmask int value.
        Returns:
            List of AccountKeyTypeFlagsDto flags representing the int value.
        """
        results = []
        for flag in AccountKeyTypeFlagsDto:
            if 0 != flag.value & bitMaskValue:
                results.append(flag)
        return results

    @classmethod
    def flagsToInt(cls, flags: List[AccountKeyTypeFlagsDto]) -> int:
        """Converts a list of AccountKeyTypeFlagsDto to a bit representation.
        Args:
            List of AccountKeyTypeFlagsDto flags representing the int value.
        Returns:
            int value of the list of flags
        """
        result = 0
        for flag in AccountKeyTypeFlagsDto:
            if flag in flags:
                result += flag.value
        return result

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.value, 1))
        return bytes_



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AmountDto import AmountDto
from .MosaicIdDto import MosaicIdDto

class MosaicBuilder:
    """Binary layout for a mosaic.

    Attributes:
        mosaicId: Mosaic identifier.
        amount: Mosaic amount.
    """

    def __init__(self, mosaicId: MosaicIdDto, amount: AmountDto):
        """Constructor.
        Args:
            mosaicId: Mosaic identifier.
            amount: Mosaic amount.
        """
        self.mosaicId = mosaicId
        self.amount = amount


    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicBuilder:
        """Creates an instance of MosaicBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicBuilder.
        """
        bytes_ = bytes(payload)

        mosaicId = MosaicIdDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[mosaicId.get_size():]
        amount = AmountDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[amount.get_size():]
        return MosaicBuilder(mosaicId, amount)

    def get_mosaic_id(self) -> MosaicIdDto:
        """Gets mosaic identifier.
        Returns:
            Mosaic identifier.
        """
        return self.mosaicId

    def get_amount(self) -> AmountDto:
        """Gets mosaic amount.
        Returns:
            Mosaic amount.
        """
        return self.amount

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.mosaicId.get_size()
        size += self.amount.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.mosaicId.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.amount.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
from typing import List
import re
from .GeneratorUtils import GeneratorUtils
from .AmountDto import AmountDto
from .EntityTypeDto import EntityTypeDto
from .KeyDto import KeyDto
from .NetworkTypeDto import NetworkTypeDto
from .SignatureDto import SignatureDto
from .TimestampDto import TimestampDto
from .TransactionBuilder import TransactionBuilder
from .TransferTransactionBodyBuilder import TransferTransactionBodyBuilder
from .UnresolvedAddressDto import UnresolvedAddressDto
from .UnresolvedMosaicBuilder import UnresolvedMosaicBuilder

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class TransferTransactionBuilder(TransactionBuilder):
    """Binary layout for a non-embedded transfer transaction.

    Attributes:
        body: Transfer transaction body.
    """
    type_hints = {
        'signature' : 'SignatureDto',
        'signer_public_key' : 'KeyDto',
        'network' : 'NetworkTypeDto',
        'type' : 'EntityTypeDto',
        'fee' : 'AmountDto',
        'deadline' : 'TimestampDto',
        'recipient_address' : 'UnresolvedAddressDto',
        'mosaics' : 'array',
    }

    VERSION = 1
    ENTITY_TYPE = 0x4154

    def __init__(self, signer_public_key, network: NetworkTypeDto):
        """Constructor.
        Args:
            signer_public_key: Entity signer's public key.
            network: Entity network.
        """
        super().__init__(signer_public_key, self.VERSION, network, self.ENTITY_TYPE)

        self.body = TransferTransactionBodyBuilder()

    @classmethod
    def load_from_binary(cls, payload: bytes) -> TransferTransactionBuilder:
        """Creates an instance of TransferTransactionBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of TransferTransactionBuilder.
        """
        bytes_ = bytes(payload)
        superObject = TransactionBuilder.load_from_binary(bytes_)
        assert cls.VERSION == superObject.version, 'Invalid entity version ({})'.format(superObject.version)
        assert cls.ENTITY_TYPE == superObject.type, 'Invalid entity type ({})'.format(superObject.type)
        bytes_ = bytes_[superObject.get_size():]

        body = TransferTransactionBodyBuilder.load_from_binary(bytes_)  # kind:CUSTOM1_nonbyte
        bytes_ = bytes_[body.get_size():]

        # create object and call
        result = TransferTransactionBuilder(superObject.signer_public_key, superObject.network)
        result.signature = superObject.signature
        result.fee = superObject.fee
        result.deadline = superObject.deadline
        result.body = body
        return result

    @property
    def recipient_address(self):
        return self.body.recipient_address

    @recipient_address.setter
    def recipient_address(self, recipient_address): # MARKER1 AttributeKind.CUSTOM
        self.body.recipient_address = recipient_address

    @property
    def mosaics(self):
        return self.body.mosaics

    @property
    def message(self):
        return self.body.message

    @message.setter
    def message(self, message): # MARKER1 AttributeKind.BUFFER
        self.body.message = message

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.body.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.body.serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += super().__str__()
        result += self.body.__str__()
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .ReceiptTypeDto import ReceiptTypeDto

class ReceiptBuilder:
    """Binary layout for a receipt entity.

    Attributes:
        version: Receipt version.
        type: Receipt type.
    """

    def __init__(self, version: int, type: ReceiptTypeDto):
        """Constructor.
        Args:
            version: Receipt version.
            type: Receipt type.
        """
        self.version = version
        self.type = type


    @classmethod
    def load_from_binary(cls, payload: bytes) -> ReceiptBuilder:
        """Creates an instance of ReceiptBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of ReceiptBuilder.
        """
        bytes_ = bytes(payload)

        size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        version = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 2))  # kind:SIMPLE
        bytes_ = bytes_[2:]
        type = ReceiptTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[type.get_size():]
        return ReceiptBuilder(version, type)

    def get_version(self) -> int:
        """Gets receipt version.
        Returns:
            Receipt version.
        """
        return self.version

    def get_type(self) -> ReceiptTypeDto:
        """Gets receipt type.
        Returns:
            Receipt type.
        """
        return self.type

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 4  # size
        size += 2  # version
        size += self.type.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        # Ignored serialization: size AttributeKind.SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_version(), 2))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.type.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .MosaicIdDto import MosaicIdDto
from .NamespaceAliasTypeDto import NamespaceAliasTypeDto

class NamespaceAliasBuilder:
    """Binary layout for alias.

    Attributes:
        namespaceAliasType: Namespace alias type.
        mosaicAlias: Mosaic alias.
        addressAlias: Address alias.
    """

    def __init__(self, namespaceAliasType: NamespaceAliasTypeDto, mosaicAlias: MosaicIdDto, addressAlias: AddressDto):
        """Constructor.
        Args:
            namespaceAliasType: Namespace alias type.
            mosaicAlias: Mosaic alias.
            addressAlias: Address alias.
        """
        self.namespaceAliasType = namespaceAliasType
        self.mosaicAlias = mosaicAlias
        self.addressAlias = addressAlias


    @classmethod
    def load_from_binary(cls, payload: bytes) -> NamespaceAliasBuilder:
        """Creates an instance of NamespaceAliasBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of NamespaceAliasBuilder.
        """
        bytes_ = bytes(payload)

        namespaceAliasType = NamespaceAliasTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[namespaceAliasType.get_size():]
        mosaicAlias = None
        if namespaceAliasType == NamespaceAliasTypeDto.MOSAIC_ID:
            mosaicAlias = MosaicIdDto.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[mosaicAlias.get_size():]
        addressAlias = None
        if namespaceAliasType == NamespaceAliasTypeDto.ADDRESS:
            addressAlias = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[addressAlias.get_size():]
        return NamespaceAliasBuilder(namespaceAliasType, mosaicAlias, addressAlias)

    def get_namespace_alias_type(self) -> NamespaceAliasTypeDto:
        """Gets namespace alias type.
        Returns:
            Namespace alias type.
        """
        return self.namespaceAliasType

    def get_mosaic_alias(self) -> MosaicIdDto:
        """Gets mosaic alias.
        Returns:
            Mosaic alias.
        """
        if not self.namespaceAliasType == NamespaceAliasTypeDto.MOSAIC_ID:
            raise Exception('namespaceAliasType is not set to MOSAIC_ID.')
        return self.mosaicAlias

    def get_address_alias(self) -> AddressDto:
        """Gets address alias.
        Returns:
            Address alias.
        """
        if not self.namespaceAliasType == NamespaceAliasTypeDto.ADDRESS:
            raise Exception('namespaceAliasType is not set to ADDRESS.')
        return self.addressAlias

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.namespaceAliasType.get_size()
        if self.namespaceAliasType == NamespaceAliasTypeDto.MOSAIC_ID:
            size += self.mosaicAlias.get_size()
        if self.namespaceAliasType == NamespaceAliasTypeDto.ADDRESS:
            size += self.addressAlias.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.namespaceAliasType.serialize())  # kind:CUSTOM
        if self.namespaceAliasType == NamespaceAliasTypeDto.MOSAIC_ID:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.mosaicAlias.serialize())  # kind:CUSTOM
        if self.namespaceAliasType == NamespaceAliasTypeDto.ADDRESS:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.addressAlias.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .AccountRestrictionsInfoBuilder import AccountRestrictionsInfoBuilder
from .AddressDto import AddressDto
from .StateHeaderBuilder import StateHeaderBuilder

class AccountRestrictionsBuilder(StateHeaderBuilder):
    """Binary layout for account restrictions.

    Attributes:
        address: Address on which restrictions are placed.
        restrictions: Account restrictions.
    """

    def __init__(self, version: int, address: AddressDto, restrictions: List[AccountRestrictionsInfoBuilder]):
        """Constructor.
        Args:
            version: Serialization version.
            address: Address on which restrictions are placed.
            restrictions: Account restrictions.
        """
        super().__init__(version)
        self.address = address
        self.restrictions = restrictions


    @classmethod
    def load_from_binary(cls, payload: bytes) -> AccountRestrictionsBuilder:
        """Creates an instance of AccountRestrictionsBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AccountRestrictionsBuilder.
        """
        bytes_ = bytes(payload)
        superObject = StateHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        address = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[address.get_size():]
        restrictionsCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIZE_FIELD
        bytes_ = bytes_[8:]
        restrictions: List[AccountRestrictionsInfoBuilder] = []  # kind:ARRAY
        for _ in range(restrictionsCount):
            item = AccountRestrictionsInfoBuilder.load_from_binary(bytes_)
            restrictions.append(item)
            bytes_ = bytes_[item.get_size():]
        return AccountRestrictionsBuilder(superObject.version, address, restrictions)

    def get_address(self) -> AddressDto:
        """Gets address on which restrictions are placed.
        Returns:
            Address on which restrictions are placed.
        """
        return self.address

    def get_restrictions(self) -> List[AccountRestrictionsInfoBuilder]:
        """Gets account restrictions.
        Returns:
            Account restrictions.
        """
        return self.restrictions

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.address.get_size()
        size += 8  # restrictionsCount
        for _ in self.restrictions:
            size += _.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.address.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(len(self.get_restrictions()), 8))  # kind:SIZE_FIELD
        for _ in self.restrictions: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .EntityTypeDto import EntityTypeDto
from .KeyDto import KeyDto
from .NetworkTypeDto import NetworkTypeDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class EmbeddedTransactionBuilder:
    """Binary layout for an embedded transaction.

    Attributes:
        signer_public_key: Entity signer's public key.
        version: Entity version.
        network: Entity network.
        type: Entity type.
    """
    type_hints = {
        'signer_public_key' : 'KeyDto',
        'network' : 'NetworkTypeDto',
        'type' : 'EntityTypeDto',
    }

    def __init__(self, signer_public_key, version, network: NetworkTypeDto, type):
        """Constructor.
        Args:
            signer_public_key: Entity signer's public key.
            version: Entity version.
            network: Entity network.
            type: Entity type.

        """
        self.signer_public_key = signer_public_key
        self.version = version
        self.network = network
        self.type = type


    @classmethod
    def load_from_binary(cls, payload: bytes) -> EmbeddedTransactionBuilder:
        """Creates an instance of EmbeddedTransactionBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of EmbeddedTransactionBuilder.
        """
        bytes_ = bytes(payload)

        size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        embedded_transaction_header__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        signer_public_key_ = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        signer_public_key = signer_public_key_.key
        bytes_ = bytes_[signer_public_key_.get_size():]
        entity_body__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        version = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]
        network_ = NetworkTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        network = network_.value
        bytes_ = bytes_[network_.get_size():]
        type_ = EntityTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        type = type_.value
        bytes_ = bytes_[type_.get_size():]

        # create object and call
        result = EmbeddedTransactionBuilder(signer_public_key, version, network, type)
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 4  # size
        size += 4  # embedded_transaction_header__reserved1
        size += KeyDto(self.signer_public_key).get_size()
        size += 4  # entity_body__reserved1
        size += 1  # version
        size += NetworkTypeDto(self.network).get_size()
        size += EntityTypeDto(self.type).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_size(), 4))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, KeyDto(self.signer_public_key).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.version, 1))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, NetworkTypeDto(self.network).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, EntityTypeDto(self.type).serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('size', to_hex_string(GeneratorUtils.uint_to_buffer(self.get_size(), 4)))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        result += '{:24s} : {}\n'.format('signer_public_key', to_hex_string(KeyDto(self.signer_public_key).serialize()))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        result += '{:24s} : {}\n'.format('version', to_hex_string(GeneratorUtils.uint_to_buffer(self.version, 1)))
        result += '{:24s} : {}\n'.format('network', to_hex_string(NetworkTypeDto(self.network).serialize()))
        result += '{:24s} : {}\n'.format('type', to_hex_string(EntityTypeDto(self.type).serialize()))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .AmountDto import AmountDto
from .BlockFeeMultiplierDto import BlockFeeMultiplierDto
from .BlockHeaderBuilder import BlockHeaderBuilder
from .DifficultyDto import DifficultyDto
from .EntityTypeDto import EntityTypeDto
from .Hash256Dto import Hash256Dto
from .HeightDto import HeightDto
from .ImportanceBlockFooterBuilder import ImportanceBlockFooterBuilder
from .KeyDto import KeyDto
from .NetworkTypeDto import NetworkTypeDto
from .SignatureDto import SignatureDto
from .TimestampDto import TimestampDto
from .VrfProofBuilder import VrfProofBuilder

class ImportanceBlockHeaderBuilder(BlockHeaderBuilder):
    """Binary layout for an importance block header.

    Attributes:
        importanceBlockFooter: Importance block footer.
    """

    def __init__(self, signature: SignatureDto, signerPublicKey: KeyDto, version: int, network: NetworkTypeDto, type: EntityTypeDto, height: HeightDto, timestamp: TimestampDto, difficulty: DifficultyDto, generationHashProof: VrfProofBuilder, previousBlockHash: Hash256Dto, transactionsHash: Hash256Dto, receiptsHash: Hash256Dto, stateHash: Hash256Dto, beneficiaryAddress: AddressDto, feeMultiplier: BlockFeeMultiplierDto, votingEligibleAccountsCount: int, harvestingEligibleAccountsCount: int, totalVotingBalance: AmountDto, previousImportanceBlockHash: Hash256Dto):
        """Constructor.
        Args:
            signature: Entity signature.
            signerPublicKey: Entity signer's public key.
            version: Entity version.
            network: Entity network.
            type: Entity type.
            height: Block height.
            timestamp: Number of milliseconds elapsed since creation of nemesis block.
            difficulty: Block difficulty.
            generationHashProof: Generation hash proof.
            previousBlockHash: Previous block hash.
            transactionsHash: Hash of the transactions in this block.
            receiptsHash: Hash of the receipts generated by this block.
            stateHash: Hash of the global chain state at this block.
            beneficiaryAddress: Beneficiary address designated by harvester.
            feeMultiplier: Fee multiplier applied to block transactions.
            votingEligibleAccountsCount: Number of voting eligible accounts.
            harvestingEligibleAccountsCount: Number of harvesting eligible accounts.
            totalVotingBalance: Total balance eligible for voting.
            previousImportanceBlockHash: Previous importance block hash.
        """
        super().__init__(signature, signerPublicKey, version, network, type, height, timestamp, difficulty, generationHashProof, previousBlockHash, transactionsHash, receiptsHash, stateHash, beneficiaryAddress, feeMultiplier)
        self.importanceBlockFooter = ImportanceBlockFooterBuilder(votingEligibleAccountsCount, harvestingEligibleAccountsCount, totalVotingBalance, previousImportanceBlockHash)


    @classmethod
    def load_from_binary(cls, payload: bytes) -> ImportanceBlockHeaderBuilder:
        """Creates an instance of ImportanceBlockHeaderBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of ImportanceBlockHeaderBuilder.
        """
        bytes_ = bytes(payload)
        superObject = BlockHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        importanceBlockFooter = ImportanceBlockFooterBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[importanceBlockFooter.get_size():]
        return ImportanceBlockHeaderBuilder(superObject.signature, superObject.signerPublicKey, superObject.version, superObject.network, superObject.type, superObject.height, superObject.timestamp, superObject.difficulty, superObject.generationHashProof, superObject.previousBlockHash, superObject.transactionsHash, superObject.receiptsHash, superObject.stateHash, superObject.beneficiaryAddress, superObject.feeMultiplier, importanceBlockFooter.votingEligibleAccountsCount, importanceBlockFooter.harvestingEligibleAccountsCount, importanceBlockFooter.totalVotingBalance, importanceBlockFooter.previousImportanceBlockHash)

    def get_voting_eligible_accounts_count(self) -> int:
        """Gets number of voting eligible accounts.
        Returns:
            Number of voting eligible accounts.
        """
        return self.importanceBlockFooter.get_voting_eligible_accounts_count()

    def get_harvesting_eligible_accounts_count(self) -> int:
        """Gets number of harvesting eligible accounts.
        Returns:
            Number of harvesting eligible accounts.
        """
        return self.importanceBlockFooter.get_harvesting_eligible_accounts_count()

    def get_total_voting_balance(self) -> AmountDto:
        """Gets total balance eligible for voting.
        Returns:
            Total balance eligible for voting.
        """
        return self.importanceBlockFooter.get_total_voting_balance()

    def get_previous_importance_block_hash(self) -> Hash256Dto:
        """Gets previous importance block hash.
        Returns:
            Previous importance block hash.
        """
        return self.importanceBlockFooter.get_previous_importance_block_hash()

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.importanceBlockFooter.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.importanceBlockFooter.serialize())  # kind:CUSTOM
        return bytes_
    # end of class
